#region License
// Copyright (c) 2005 Alexander Kolliopoulos
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would
//    be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must not
//    be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
#endregion

using System;
using System.Runtime.Serialization;

namespace Lydos.Matlab
{
    #region MatlabException
    /// <summary>
    /// The base class for exceptions caused by Matlab errors.
    /// </summary>
    [Serializable]
    public class MatlabException : Exception
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabBadTypeException
    /// <summary>
    /// The exception that is thrown when attempting to use a Matrix with a non-double Matlab type.
    /// </summary>
    [Serializable]
    public class MatlabBadTypeException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabBadTypeException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabBadTypeException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabBadTypeException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabBadTypeException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabDeleteVariableFailedException
    /// <summary>
    /// The exception that is thrown when a function failed to delete a Matlab variable.
    /// </summary>
    [Serializable]
    public class MatlabDeleteVariableFailedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabDeleteVariableFailedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabDeleteVariableFailedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabDeleteVariableFailedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabDeleteVariableFailedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabGetVariableFailedException
    /// <summary>
    /// The exception that is thrown when a function failed to get a variable from Matlab.
    /// </summary>
    [Serializable]
    public class MatlabGetVariableFailedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabGetVariableFailedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabGetVariableFailedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabGetVariableFailedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabGetVariableFailedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabNotConnectedException
    /// <summary>
    /// The exception that is thrown when attempting to use Matlab without an open connection.
    /// </summary>
    [Serializable]
    public class MatlabNotConnectedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabNotConnectedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabNotConnectedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabNotConnectedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabNotConnectedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabOutOfMemoryException
    /// <summary>
    /// The exception that is thrown when Matlab failed because it ran out of unmanaged memory.
    /// </summary>
    [Serializable]
    public class MatlabOutOfMemoryException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabOutOfMemoryException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabOutOfMemoryException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabOutOfMemoryException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabOutOfMemoryException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabPutVariableFailedException
    /// <summary>
    /// The exception that is thrown when a function failed to set a variable in Matlab.
    /// </summary>
    [Serializable]
    public class MatlabPutVariableFailedException : Exception
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabPutVariableFailedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabPutVariableFailedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabPutVariableFailedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabPutVariableFailedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion

    #region MatlabUnspecifiedException
    /// <summary>
    /// The exception that is thrown when Matlab failed, but the reason is not clear.
    /// </summary>
    [Serializable]
    public class MatlabUnspecifiedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabUnspecifiedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabUnspecifiedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabUnspecifiedException(string message, Exception inner) : base(message, inner) { }
 
        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabUnspecifiedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }
    #endregion
}
